/*
 * LCD.c
 *
 * Created: 23.4.2013 13:07:37
 *  Author: mjankovec
 */ 

#include "LCD.h"

FILE LCD_str = FDEV_SETUP_STREAM(LCD_putc,NULL,_FDEV_SETUP_WRITE);

char lcd_x, lcd_y;

int LCD_putc(char c, FILE *stream)
{
	switch (c)
	{
		case '\r':	LCD_GotoXY(0,lcd_y);
					break;
		case '\n':	LCD_GotoXY(lcd_x,1-lcd_y);
					break;
		default:
			LCD_Write(Data,c);
			lcd_x++;
			if (lcd_x >= 16) LCD_GotoXY(0,1-lcd_y);
			break;
	}
	return 0;
}

//************************************
// Method:    LCD_Busy
// FullName:  LCD_Busy
// Access:    public 
// Returns:   char
// Qualifier: Funkcija vrne 0, ?e je LCD pripravljen za vpis podatka
//************************************

char LCD_Busy()
{
	char data;
	
	//D4 - D7 = PORTA 4-7
	//RS(PC7) = 0, RW(PC6) = 1, E(PC0)
	LCD_IR();
	LCD_R();
	
	//Tu beremo zgornje 4 bite
	LCD_E_set();
	asm("nop");
	asm("nop");
	data = LCD_DataPin; //preberi zgornje stiri bite
	LCD_E_clr();
	asm("nop");
	asm("nop");
	
	//Tu beremo spodnje 4 bite (samo da jih v bistvu ignoriramo)
	LCD_E_set();
	asm("nop");
	asm("nop");
	LCD_E_clr();
	
	if (data & 0x80)	return 1;	// pomeni da je zaseden
	else return 0;	// pomeni da ni zaseden
}



//************************************
// Method:    LCD_BusyWait
// FullName:  LCD_BusyWait
// Access:    public 
// Returns:   void
// Qualifier: funkcija ?aka, dokler ni LCD pripravljen ali dokler ne pote?e timeout. Timeout je odvisen od ure procesorja.
// Parameter: unsigned int timeout
//************************************
void LCD_BusyWait(unsigned int timeout)
{
	for (unsigned int i=0;i<timeout;i++)
	{
		if (!LCD_Busy()) break;
	}	
}

//************************************
// Method:    LCD_Write
// FullName:  LCD_Write
// Access:    public 
// Returns:   void
// Qualifier: Funkcija vpie podatek v LCD
// Parameter: enum LCD_Register reg
// Parameter: char data
//************************************
void LCD_Write(enum LCD_Register reg, char data)
{
	char temp;
	
	LCD_BusyWait(1000);
	
	//D4 - D7 = PORTA 4-7

	if (reg == Instr) LCD_IR(); // pii v intrukcijski register
	else LCD_DR(); // pii v podatkovni register
	
	LCD_W();
	LCD_E_set();
	
	// write high nibble
	temp = data>>4;	 // dddd 0000 -> 0000 dddd
	LCD_Write_Nibble(temp);

	LCD_E_clr();
	asm("nop");
	asm("nop");
	
	LCD_E_set();
	
	// write low nibble
	temp = data & 0x0F;	 //  0000 dddd
	LCD_Write_Nibble(temp);
	
	LCD_E_clr();
}

//************************************
// Method:    LCD_Init
// FullName:  LCD_Init
// Access:    public 
// Returns:   void
// Qualifier: Funkcija inicializira LCD
//************************************
void LCD_Init()
{
	
	//nastavi ustrezne pine kot izhode
	LCD_E_Dir |= LCD_E_Mask;
	LCD_RS_Dir |= LCD_RS_Mask;
	LCD_RW_Dir |= LCD_RW_Mask;
	LCD_DataDir |= LCD_DataMask;
	LCD_Backlight_Dir |= LCD_Backlight_Mask;
	
	LCD_Backlight_ON();
	
	// tukaj vpiite startup sekvenco za incializaicjo LCD (slide 25)
	_delay_ms(15);
	LCD_IR();
	LCD_W();
	LCD_E_set();
	LCD_Write_Nibble(0x03);
	LCD_E_clr();
	_delay_ms(5);
	LCD_E_set();
	LCD_Write_Nibble(0x03);
	LCD_E_clr();
	_delay_us(120);
	LCD_E_set();
	LCD_Write_Nibble(0x03);
	LCD_E_clr();
	_delay_us(120);
	LCD_E_set();
	LCD_Write_Nibble(0x02);
	LCD_E_clr();
	
	LCD_BusyWait(100);
	LCD_Write(Instr, 0x28);
	LCD_BusyWait(100);
	LCD_Write(Instr, 0x08);
	LCD_BusyWait(100);
	LCD_Write(Instr, 0x01);
	LCD_BusyWait(100);
	LCD_Write(Instr, 0x06);
	LCD_BusyWait(100);
	LCD_Write(Instr, 0x0F);
}

void LCD_GotoXY(char col, char row)
{
	char naslov;
	if (col > 15) return;
	if (row > 1) return;
	lcd_x=col;
	lcd_y=row;
	naslov = col + 0x40*row;
	LCD_Write(Instr, 0x80 | naslov);
}

void LCD_WriteString(char *str)
{
	// *str - kar je na naslovu str
	// &a - naslov od a
	int i;
	
	for ( i=0 ; str[i] ; i++)
	{
		LCD_Write(Data,str[i]);
		lcd_x++;
	}
}

void LCD_WriteBytes(char *data, char count)
{
	// *str - kar je na naslovu str
	// &a - naslov od a
	int i;
	
	for ( i=0 ; i<count ; i++)
	{
		LCD_Write(Data,data[i]);
		lcd_x++;
	}
}