/*
 * UART1_IRQ.c
 *
 * Created: 27.3.2015 12:28:18
 *  Author: maticpi
 */ 

#include "UART1_IRQ.h"

FILE UART1_str = FDEV_SETUP_STREAM(UART1_putc,NULL,_FDEV_SETUP_WRITE);

volatile char TxBuf1[TB_SIZE1];
int TBin1;
volatile int TBout1;
volatile int TBnum1;

volatile char RxBuf1[RB_SIZE1];
int RBin1;
volatile int RBout1;
volatile int RBnum1;

char UART1_SendStr(char str[])
{
	int i;
	for (i=0; str[i]; i++)
	{
		if (!UART1_put(str[i])) return i;
	}
	return i;
}

char UART1_SendBytes(char data[], int num)
{
	int i;
	for (i=0; i<num; i++)
	{
		if (!UART1_put(data[i])) return i;
	}
	return i;
}

int UART1_putc(char c, FILE *stream)	//for use with printf
{
	if (UART1_put(c)) return 0;		//All OK
	else return EOF;			//Error (EOF = -1)
}

char UART1_put(char data)
{
	if (UART1_numTxBytes() >= TB_SIZE1) return 0;	//Error - buffer full
	ATOMIC_BLOCK(ATOMIC_RESTORESTATE)
	{
		TxBuf1[TBin1]=data;
		TBnum1++;
	}
	TBin1++;
	if (TBin1 >= TB_SIZE1) TBin1=0;
	UCSR1B |= (1<<UDRIE1);	//enable transmit IRQ
	return 1;	//all OK
}

char UART1_getTxByteFromBuffer()	//Called only in interrupts (if present), so it doesn't need atomic blocks
{
	char tmp;
	if (UART1_numTxBytes() <= 0) return 0;	//Error - nothing to return (although 0 is also a valid data)
	tmp=TxBuf1[TBout1];
	TBnum1--;
	TBout1++;
	if (TBout1 >= TB_SIZE1) TBout1=0;
	return tmp;
}

int UART1_numTxBytes()
{
	int tmp;
	ATOMIC_BLOCK(ATOMIC_RESTORESTATE) { tmp=TBnum1; }
	return tmp;
}

int UART1_numRxBytes()
{
	int tmp;
	ATOMIC_BLOCK(ATOMIC_RESTORESTATE) { tmp=RBnum1; }
	return tmp;
}

int UART1_DataReady()
{
	return (UART1_numRxBytes() > 0);
}

char UART1_GetByte()
{
	char data=0;
	ATOMIC_BLOCK(ATOMIC_RESTORESTATE)
	{
		if (RBnum1 > 0)
		{
			data=RxBuf1[RBout1];
			RBout1++;
			if (RBout1 >= RB_SIZE1) RBout1=0;
			RBnum1--;
		}
	}
	return data;
}

char UART1_PreviewRxByte(int index)
{
	char data=0;
	index += RBout1;
	while (index >= RB_SIZE1) index -= RB_SIZE1;
	ATOMIC_BLOCK(ATOMIC_RESTORESTATE)
	{
		data=RxBuf1[index];
	}
	return data;
}

ISR(USART1_RX_vect)
{
	if (RBnum1 < RB_SIZE1)
	{
		RxBuf1[RBin1]=UDR1;
		RBin1++;
		if (RBin1 >= RB_SIZE1) RBin1=0;
		RBnum1++;
	}
}

ISR(USART1_UDRE_vect)
{
	if (TBnum1 > 0)
	{
		UDR1=TxBuf1[TBout1];
		TBout1++;
		if (TBout1 >= TB_SIZE1) TBout1=0;
		TBnum1--;
	}
	else
	{
		UCSR1B &= ~(1<<UDRIE1);		//izklopi prekinitev za oddajo
	}
}

void UART1_Init()
{
	//1-output, 0-input
	DDRD |= (1<<3); //PD3 - TX1
	DDRD &= ~(1<<2); //PD2 - RX1
	
	//USART1 - UART1_ 115200 baud, 8bits, 1 stop bit, no parity, no flow control
	UCSR1A = 0;
	UCSR1C = (0<<UMSEL10) | (0<<UPM10) | (0<<USBS1) | (3<<UCSZ10);
	UCSR1B = (1<<RXEN1) | (1<<TXEN1) | (1<<RXCIE1) | (0<<UCSZ12);

	#ifndef BAUD1
		#warning "UART1_IRQ.c: BAUD1 not defined! Assuming 115200."
		#define BAUD1 115200UL
	#endif
	#undef BAUD
	#define BAUD BAUD1
	#include <util/setbaud.h>
	UBRR1H = UBRRH_VALUE;
	UBRR1L = UBRRL_VALUE;
	#if USE_2X
	UCSR1A |= (1 << U2X1);
	#else
	UCSR1A &= ~(1 << U2X1);
	#endif
}