/*
 * TLC59401.h
 *
 * Created: 12-Jul-16 15:07:15
 *  Author: maticpi
 */ 


#ifndef TLC59401_H_
#define TLC59401_H_

#include <avr/io.h>
#include <avr/eeprom.h>
#include "config.h"
#include <util/delay.h>
#include <stdio.h>

//GSCLK - gray scale PWM
#define TLC59401_GSCLK_DDR	DDRD
#define TLC59401_GSCLK_PORT	PORTD
#define TLC59401_GSCLK_PIN	PIND
#define TLC59401_GSCLK_BIT	6

//BLANK - output (0: Outputs ON, 1: Outputs OFF)
#define TLC59401_BLANK_DDR	DDRD
#define TLC59401_BLANK_PORT	PORTD
#define TLC59401_BLANK_PIN	PIND
#define TLC59401_BLANK_BIT	5

#define TLC59401_SPI_SCK_DELAY()	_delay_us(1)

//XLAT - output (0: don't latch, 1: latch)
#define	TLC59401_XLAT_DDR	DDRD
#define	TLC59401_XLAT_PORT	PORTD
#define TLC59401_XLAT_PIN	PIND
#define	TLC59401_XLAT_BIT	4
#define TLC59401_LATCH_DELAY()	_delay_us(1)

//MODE - output (0: GrayScale, 1: DotCorrection)
#define TLC59401_MODE_DDR	DDRB
#define TLC59401_MODE_PORT	PORTB
#define TLC59401_MODE_PIN	PINB
#define TLC59401_MODE_BIT	2

//ERR1,2 - inputs (0: Error, 1: No error)
#define TLC59401_ERR1_DDR	DDRC
#define TLC59401_ERR1_PORT	PORTC
#define TLC59401_ERR1_PIN	PINC
#define TLC59401_ERR1_BIT	3

#define TLC59401_ERR2_DDR	DDRB
#define TLC59401_ERR2_PORT	PORTB
#define TLC59401_ERR2_PIN	PINB
#define TLC59401_ERR2_BIT	3	//Originally PB4, but that is the SS pin which has side-effects if SPI is in operation

#define GSClkHigh()	TLC59401_GSCLK_PORT |= (1<<TLC59401_GSCLK_BIT)
#define GSClkLow()	TLC59401_GSCLK_PORT &= ~(1<<TLC59401_GSCLK_BIT)
#define GSClkPulse()	GSClkHigh(); GSClkLow()
#define LatchOn()	TLC59401_XLAT_PORT |= (1<<TLC59401_XLAT_BIT)
#define LatchOff()	TLC59401_XLAT_PORT &= ~(1<<TLC59401_XLAT_BIT)
#define ModeGS()	TLC59401_MODE_PORT &= ~(1<<TLC59401_MODE_BIT)
#define ModeDC()	TLC59401_MODE_PORT |=  (1<<TLC59401_MODE_BIT)
#define OutputsOn()		TLC59401_BLANK_PORT &= ~(1<<TLC59401_BLANK_BIT)
#define OutputsOff()	TLC59401_BLANK_PORT |=  (1<<TLC59401_BLANK_BIT)

#define IsErr1()	(!(TLC59401_ERR1_PIN & (1<<TLC59401_ERR1_BIT)))
#define IsErr2()	(!(TLC59401_ERR2_PIN & (1<<TLC59401_ERR2_BIT)))

void TLC59401_Init();
void SetGSandGetStatus(uint16_t* data, uint8_t* status);
void SetDC(uint8_t* data);
void GetDataFromEEPROM();
void SaveDataToEEPROM();
void PulseSck();
void EnablePWMmode();
void DisablePWMmode();
void ParseDCdata(uint8_t* data);
void ParseCalData();
void EnableLockIn();
void DisableLockIn();
void ParseSetCorrFactorCommand();

extern uint16_t GSdata[16*TLC59401_COUNT];
extern uint8_t DCdata10[16*TLC59401_COUNT];
extern uint8_t DCdata30[16*TLC59401_COUNT];
extern uint8_t DCdata100[16*TLC59401_COUNT];
extern float CalData[3];	//0- Low, 2-High
extern uint8_t status[24*TLC59401_COUNT];
extern uint8_t LEDStatus;
#define LEDS_ON10				0x01
#define LEDS_ON30				0x02
#define LEDS_ON100				0x03
#define LEDS_OFF				0x04
#define LEDS_ON10_OVERTEMP		0x07
#define LEDS_ON30_OVERTEMP		0x08
#define LEDS_ON100_OVERTEMP		0x09
#define LEDS_NO_POWER			0x0E
#define LEDS_ERROR				0x0F

extern uint8_t CycleTypeStatus;
#define LAST_CYCLE_DC	1
#define LAST_CYCLE_GS	0

extern uint8_t PWM;
#endif /* TLC59401_H_ */