/*
 * usage_statistics.c
 *
 * Created: 24-Jul-16 20:51:43
 *  Author: maticpi
 */ 

#include "usage_statistics.h"

uint32_t usageCounter;
uint32_t backupUsageCounter;
uint32_t elapsedTime;

void InitUsageCounters()
{
	usageCounter = eeprom_read_dword ((uint32_t *)EEPROM_ADDRESS_USAGE);
	if (usageCounter == 0xFFFFFFFF)
	{
		eeprom_write_dword((uint32_t *)EEPROM_ADDRESS_USAGE, 0);
		usageCounter=0;
	}
	CheckBackupUsageData();
}

void StoreUsageData(uint8_t invalidate)
{
	eeprom_write_dword((uint32_t *)EEPROM_ADDRESS_USAGE, usageCounter);
	backupUsageCounter=0;
	if (invalidate) InvalidateBackedupUsageData();
}
void UpdateUsage()
{
	static uint32_t timeMarkUsage=0;
	//	static uint32_t usageMinCounter=0;
	
	if (Has_X_MillisecondsPassed(1000,&timeMarkUsage))
	{
		if ((LEDStatus == LEDS_ON10) || (LEDStatus == LEDS_ON30) || (LEDStatus == LEDS_ON100))
		{
			elapsedTime++;
			usageCounter++;
			backupUsageCounter++;
			if (backupUsageCounter/60 > 0)	//store usage backup  every minute
			{
				backupUsageCounter=0;
				SaveBackupUsageData();
			}
		}
	}
}

void SaveBackupUsageData()	//+1 min shrani v EEPROM (1 bit)
{
	// 	eeprom_write_ and eeprom_update_ both use erase and write-> no good.
	// 	Use write only to reduce EEPROM wear - do it manually
	//	Wear-leveling: use circular buffer, say 8 bytes for starters, half of EEPROM for final version. E
	//		Each byte has a guard bit - bit0: 1-byte is still valid,
	//										  0-byte has already been used and has to be erased (when we get around to it again)
	//		bits 1 to 7: start from 1, go to 7, if bit is 0, it marks a minute has passed
	//	When storing usage data, invalidate all the used bytes
	int i;
	uint8_t tmp,tmp2;

	for (i=0; i<EEPROM_TMP_USAGE_BUF_SIZE; i++)
	{
		tmp = eeprom_read_byte ((uint8_t *)(EEPROM_ADDRESS_TMP_USAGE+i));
		if ((tmp & 0x01) && (tmp & 0x80)) break;	//if byte valid and not completely used, write here
	}
	if (i>=EEPROM_TMP_USAGE_BUF_SIZE)	//No free space
	{//a je nujno da so vsi byte-i valid?- possible BUG!!!
		usageCounter+=7UL*EEPROM_TMP_USAGE_BUF_SIZE*60UL;
		StoreUsageData(0);
		//TODO: This takes a long time! Yield time to other stuff while doing this!!!
		for (i=0; i<EEPROM_TMP_USAGE_BUF_SIZE; i++)
		{
			EEPROM_Erase_byte((uint8_t *)(EEPROM_ADDRESS_TMP_USAGE+i));
		}
	}
	else	//found the byte to write to
	{
		if (i==0)
		{
			tmp2 = eeprom_read_byte ((uint8_t *)(EEPROM_ADDRESS_TMP_USAGE+EEPROM_TMP_USAGE_BUF_SIZE-1));
			if ((tmp2 != 0xff) && (tmp2 & 0x01) && (tmp2 & 0x80))
			{
				EEPROM_ClearFirstAvailableBit((uint8_t *)(EEPROM_ADDRESS_TMP_USAGE+EEPROM_TMP_USAGE_BUF_SIZE-1));
				if (!(tmp & 0x01)) EEPROM_Erase_byte ((uint8_t *)EEPROM_ADDRESS_TMP_USAGE);
			}
			else
			{
				EEPROM_ClearFirstAvailableBit((uint8_t *)(EEPROM_ADDRESS_TMP_USAGE));
				tmp = eeprom_read_byte ((uint8_t *)(EEPROM_ADDRESS_TMP_USAGE+1));
				if (!(tmp & 0x01)) EEPROM_Erase_byte ((uint8_t *)(EEPROM_ADDRESS_TMP_USAGE+1));
			}
		}
		else
		{
			EEPROM_ClearFirstAvailableBit((uint8_t *)(EEPROM_ADDRESS_TMP_USAGE+i));
			//prepare next byte
			i = (i+1) % EEPROM_TMP_USAGE_BUF_SIZE;
			tmp = eeprom_read_byte ((uint8_t *)(EEPROM_ADDRESS_TMP_USAGE+i));
			if (!(tmp & 0x01)) EEPROM_Erase_byte ((uint8_t *)(EEPROM_ADDRESS_TMP_USAGE+i));
		}
	}
}

uint8_t EEPROM_ClearFirstAvailableBit(uint8_t* __adr)
{
	int i;
	uint8_t data;
	
	data = eeprom_read_byte (__adr);
	for (i=1;i<8;i++)
	{
		if (data & (1<<i))
		{
			EEPROM_WriteOnly_byte(__adr, ~(1<<i));
			return 1;
		}
	}
	return 0;
}

//reads EEPROM bitfield, gathers usage data, stores to proper EEPROM variable and invalidates used bytes in EEPROM
//TODO: 
void CheckBackupUsageData() 
{
	int i,j;
	int count=0;
	int end_index=-1;//0
	int first_unused_index=-1;
	uint8_t tmp;
	uint8_t end_found=0;	//needed for wrap-around - to continue where last finished, even if the last usage
							//statistics wraps around the end of EEPROM usage bitfield
	
	for (i=0; i<EEPROM_TMP_USAGE_BUF_SIZE; i++)
	{
		tmp = eeprom_read_byte ((uint8_t *)(EEPROM_ADDRESS_TMP_USAGE+i));
		if ((tmp & 0x01) && tmp != 0xff)	//if byte valid and used
		{
			//if (tmp & 0x80) end_index=i;	//mark the index of the last used byte
			if (!end_found) end_index=i;
			//count backed up minutes
			if (tmp == 0x01) count+=7;
			else
			{
				for (j=1;j<8;j++)
				if (!(tmp & (1<<j))) count++;
			}
			//and invalidate the byte
			EEPROM_WriteOnly_byte((uint8_t *)(EEPROM_ADDRESS_TMP_USAGE+i), 0xFE);
		}
		else
		{
			if (tmp == 0xff)
				if (first_unused_index < 0) first_unused_index=i;
			if (end_index>=0) end_found=1;
		}
	}
	if (first_unused_index<0)//if no unused bytes found
	{
		//mark the first available byte
		i = (end_index + 1) % EEPROM_TMP_USAGE_BUF_SIZE;
		EEPROM_Erase_byte ((uint8_t *)(EEPROM_ADDRESS_TMP_USAGE+i));
	}
	//Update usage data 
	if (count > 0)
	{
		usageCounter+=count*60UL;
		StoreUsageData(0);
	}

}

void InvalidateBackedupUsageData()
{
	int i;
	uint8_t tmp;
	
	for (i=0; i<EEPROM_TMP_USAGE_BUF_SIZE; i++)
	{
		tmp = eeprom_read_byte ((uint8_t *)(EEPROM_ADDRESS_TMP_USAGE+i));
		if ((tmp & 0x01) && (tmp !=0xFF))	//if byte valid and used
		{
			//invalidate the byte
			EEPROM_WriteOnly_byte((uint8_t *)(EEPROM_ADDRESS_TMP_USAGE+i), 0xFE);
		}
	}
}
void EEPROM_WriteOnly_byte(uint8_t *__p, uint8_t __value)
{
	while (EECR & (1<<EEPE)) {}	//Wait for EEPE to become zero
	EEAR = (uint16_t)__p;		//set address register
	EEDR = __value;				//set data register
	
	ATOMIC_BLOCK(ATOMIC_RESTORESTATE)
	{
		EECR = (2<<EEPM0) | (1<<EEMPE);//Write only (NO ERASE), Enable Master Write enable
		EECR |= (1<<EEPE);	//Start write
	}
}

void EEPROM_Erase_byte(uint8_t *__p)
{
	while (EECR & (1<<EEPE)) {}	//Wait for EEPE to become zero
	EEAR = (uint16_t)__p;		//set address register
	EEDR = 0xff;				//set data register - not necessary
	
	ATOMIC_BLOCK(ATOMIC_RESTORESTATE)
	{
		EECR = (1<<EEPM0) | (1<<EEMPE);//Erase only (NO WRITE), Enable Master Write enable
		EECR |= (1<<EEPE);	//Start write
	}
}

/*//FOR DEBUG PURPOSES ONLY:
uint8_t EEPROM_old_img[EEPROM_TMP_USAGE_BUF_SIZE]={
	0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,
	0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,
	0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,
	0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,
	0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,
	0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,
	0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,
	0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF
	};
void PrintEEPROMusgData()
{
	int i;
	uint32_t usgCnt;
	int h,m,s;
	uint8_t data;
	for (i=0; i < EEPROM_TMP_USAGE_BUF_SIZE; i++)
	{
		if (i%16 == 0) printf("\r\n0x%02x: ",i);
		data = eeprom_read_byte ((uint8_t *)(EEPROM_ADDRESS_TMP_USAGE+i));
		if (data != EEPROM_old_img[i]) printf("\x1B[31m%02X\x1B[37m ",data);	//vt52 red   ... white
		else 
		{
			if (data == 0xFF)		printf("\x1B[32m%02X\x1B[37m ",data);	//unused green
			else if (data == 0x00)	printf("\x1B[34m%02X\x1B[37m ",data);	//completely used blue
			else if (data &  0x01)	printf("\x1B[33m%02X\x1B[37m ",data);	//used, valid yellow
			else					printf("\x1B[36m%02X\x1B[37m ",data);	//used, invalid cyan
		}
//		printf("%02X ",data);	//no colors
		EEPROM_old_img[i]=data;
	}
	usgCnt=usageCounter;
	h=usgCnt/3600UL;
	m=(usgCnt-h*3600UL)/60UL;
	s=usgCnt-h*3600UL-m*60UL;
	printf("\r\n       Usage: %02d:%02d:%02d",h,m,s);
	usgCnt=eeprom_read_dword ((uint32_t *)EEPROM_ADDRESS_USAGE);
	h=usgCnt/3600UL;
	m=(usgCnt-h*3600UL)/60UL;
	s=usgCnt-h*3600UL-m*60UL;
	printf("\r\nEEPROM Usage: %02d:%02d:%02d",h,m,s);
}

void EraseUsageEEPROM()
{
	int i;
	for (i=-4; i < EEPROM_TMP_USAGE_BUF_SIZE; i++)
	{
		EEPROM_Erase_byte((uint8_t *)(EEPROM_ADDRESS_TMP_USAGE+i));
	}
	
}
*/